from llq.clients import RestClient, GraphQLClient
from llq.queries import EventByStartDateQuery, PartnerByIdQuery, CustomTermsQuery, PartnersQuery 
from llq.mutations import UpdateJobMutation, UpdateJobStatusMutation, DeleteJobMutation, LoginMutation, RefreshTokenMutation
from llq.rest_operations import post_job

__all__ = [
    "RestClient", 
    "GraphQLClient", 
    "post_job", 
    "EventByStartDateQuery", 
    "PartnerByIdQuery",
    "CustomTermsQuery",
    "PartnersQuery",
    "LoginMutation",
    "RefreshTokenMutation",
    "UpdateJobStatusMutation",
    "UpdateJobMutation",
    "DeleteJobMutation",
]



from abc import ABC, abstractmethod


class BaseResponse(ABC):
    """
    Abstract base class for all response objects.
    """

    @staticmethod
    @abstractmethod
    def from_dict(data: dict) -> "BaseResponse":
        """
        Parse a dictionary into an instance of the response class.

        :param data: A dictionary representing the response data.
        :return: An instance of the implementing response class.
        """
        pass



from dataclasses import dataclass
from typing import Optional
from llq.type import Node, Nodes, MediaItem


@dataclass
class Organizer:
    uri: str
    title: str
    content: Optional[str]
    featured_image: Node[MediaItem]


@dataclass
class EventsCategory:
    name: str


@dataclass
class Venue:
    uri: str
    city: Optional[str]
    address: Optional[str]


@dataclass
class Event:
    uri: str
    title: str
    duration: Optional[int]
    start_date: str
    end_date: Optional[str]
    date: Optional[str]
    all_day: bool
    url: str
    link: Optional[str]
    organizers: Optional[Nodes[Organizer]]
    events_categories: Optional[Nodes[EventsCategory]]
    featured_image: Optional[Node[MediaItem]]
    venue: Optional[Venue]

    @staticmethod
    def from_dict(data: dict) -> "Event":
        return Event(
            uri=data["uri"],
            title=data["title"],
            duration=data.get("duration"),
            start_date=data["start_date"],
            end_date=data.get("end_date"),
            date=data.get("date"),
            all_day=data["all_day"],
            url=data["url"],
            link=data.get("link"),
            organizers=Nodes.from_dict(data.get("organizers", {}), Organizer)
            if data.get("organizers")
            else None,
            events_categories=Nodes.from_dict(
                data.get("events_categories", {}), EventsCategory
            )
            if data.get("events_categories")
            else None,
            featured_image=Node.from_dict(data.get("featured_image", {}), MediaItem)
            if data.get("featured_image")
            else None,
            venue=Venue(
                uri=data["venue"]["uri"],
                city=data["venue"].get("city"),
                address=data["venue"].get("address"),
            )
            if data.get("venue")
            else None,
        )


__all__ = ["Event", "Venue", "EventsCategory", "Organizer"]



from sgqlc.operation import Fragment
from sgqlc.types import Type as SGQLCType
from typing import List
from llq.schema import Event, EventsCategory, MediaItem, Venue, PartnerAcf
from llq.fragments.fragment_builder import FragmentBuilder
from llq.fragments.media_item import MediaItemFragment


class PartnerFragment(FragmentBuilder[PartnerAcf]):
    def get(self) -> Fragment:
        # Partner (ACF) Fragment
        fragment = Fragment(PartnerAcf, "PartnerFragment")
        fragment.partner_name()
        fragment.partner_description()
        fragment.partner_website_link()
        fragment.technologie()
        fragment.type_of_company()

        # Nested Fragments

        image_node = fragment.partner_logo().node
        media_item_frag = MediaItemFragment().get()
        image_node.__fragment__(media_item_frag)

        return fragment



from sgqlc.operation import Fragment
from sgqlc.types import Type as SGQLCType
from typing import List
from llq.schema import Event, EventsCategory, MediaItem, Venue
from llq.fragments.fragment_builder import FragmentBuilder
from llq.fragments.organizer import OrganizerFragment
from llq.fragments.media_item import MediaItemFragment
from llq.fragments.venue import VenueFragment


class EventFragment(FragmentBuilder[Event]):
    def get(self) -> Fragment:
        # Event Fragment
        fragment = Fragment(Event, "EventFragment")
        fragment.uri()
        fragment.title()
        fragment.duration()
        fragment.start_date()
        fragment.end_date()
        fragment.date()
        fragment.all_day()
        fragment.url()
        fragment.link()

        # Nested Fragments
        organizers = fragment.organizers
        organizer_nodes = organizers.nodes
        organizer_frag = OrganizerFragment().get()
        organizer_nodes.__fragment__(organizer_frag)

        # EventsCategories Fragment
        category_frag = self.events_category_fragment()
        events_categories = fragment.events_categories
        category_nodes = events_categories.nodes
        category_nodes.__fragment__(category_frag)

        # FeaturedImage Fragment
        featured_image = fragment.featured_image
        image_node = featured_image.node
        media_item_frag = MediaItemFragment().get()
        image_node.__fragment__(media_item_frag)

        # Venue Fragment
        venue = fragment.venue
        venue_frag = VenueFragment().get()
        venue.__fragment__(venue_frag)

        return fragment

    def events_category_fragment(self) -> Fragment:
        category_frag = Fragment(EventsCategory, "EventsCategoryFragment")
        category_frag.name()
        return category_frag



from dataclasses import dataclass
from typing import Optional


@dataclass
class MediaItem:
    uri: str
    source_url: Optional[str]
    alt_text: Optional[str]
    database_id: int
    media_item_url: str



from abc import ABC, abstractmethod
from typing import Dict
from sgqlc.operation import Operation
from llq.base_response import BaseResponse


class BaseOperationBuilder(ABC):
    """
    Abstract base class for all Query/Mutation Builders.
    """

    @abstractmethod
    def get(self, **kwargs) -> Operation:
        """
        Build a GraphQL query or mutation.

        :param kwargs: Parameters required to build the query or mutation.
        :return: An sgqlc Operation object representing the query or mutation.
        """
        pass

    @staticmethod
    @abstractmethod
    def parse(data: Dict) -> "BaseResponse":
        """
        Parse the GraphQL response data into a typed BaseResponse object.

        :param data: A dictionary representing the GraphQL response.
        :return: A BaseResponse instance.
        """
        pass



from dataclasses import dataclass
from llq.type import Node, Nodes, MediaItem
from typing import Optional


@dataclass
class JobAcf:
    job_title: str
    compagny_name: str
    localization: str
    apply_link: str


@dataclass
class OccupationKind:
    id: str
    uri: str
    name: str


@dataclass
class JobMode:
    id: str
    uri: str
    name: str


@dataclass
class ContractKind:
    id: str
    uri: str
    name: str


@dataclass
class Job:
    uri: str
    job_acf: Optional[JobAcf]
    compagny_logo: Optional[Node[MediaItem]]
    occupation_kinds: Optional[Nodes[OccupationKind]]
    job_modes: Optional[Nodes[JobMode]]
    contract_kinds: Optional[Nodes[ContractKind]]

    @staticmethod
    def from_dict(data: dict) -> "Job":
        """
        Parse a dictionary into a Job instance.
        """
        return Job(
            uri=data["uri"],
            job_acf=JobAcf(
                job_title=data["job_acf"]["job_title"],
                compagny_name=data["job_acf"]["compagny_name"],
                localization=data["job_acf"]["localization"],
                apply_link=data["job_acf"]["apply_link"],
            )
            if data.get("job_acf")
            else None,
            compagny_logo=Node.from_dict(data.get("compagny_logo", {}), MediaItem)
            if data.get("compagny_logo")
            else None,
            occupation_kinds=Nodes.from_dict(
                data.get("occupationkinds", {}), OccupationKind
            )
            if data.get("occupationkinds")
            else None,
            job_modes=Nodes.from_dict(data.get("jobmodes", {}), JobMode)
            if data.get("jobmodes")
            else None,
            contract_kinds=Nodes.from_dict(data.get("contractkinds", {}), ContractKind)
            if data.get("contractkinds")
            else None,
        )


@dataclass
class CreateJobAcf:
    job_title_: str
    job_description_: str
    job_localization_: str
    job_compagny_name_: str
    job_compagny_logo: int
    job_contact_email: Optional[str]
    job_apply_link: Optional[str]


__all__ = ["JobAcf", "OccupationKind", "JobMode", "ContractKind", "Job", "CreateJobAcf"]



from sgqlc.operation import Operation
from llq.schema import (
    RootMutation,
    ID,
    JobContractkindsInput,
    JobJobmodesInput,
    JobOccupationkindsInput,
    PostStatusEnum,
)
from llq.base_operation_builder import BaseOperationBuilder
from llq.fragments import JobFragment
from typing import Optional
from llq.type.job import Job
from dataclasses import dataclass

@dataclass
class UpdateJobMutationResponse:
    job: Optional[Job]

    @staticmethod
    def from_dict(data: dict) -> "UpdateJobMutationResponse":
        """
        Parse a dictionary into an UpdateJobMutationResponse instance.
        """
        return UpdateJobMutationResponse(
            job=Job.from_dict(data.get("job", {})) if data.get("job") else None
        )


class UpdateJobMutation(BaseOperationBuilder):
    """
    A builder for job update-related GraphQL mutation.
    """

    def get(
        self,
        job_id: ID,
        status: PostStatusEnum,
        contract_kinds: JobContractkindsInput,
        job_modes: JobJobmodesInput,
        occupation_kinds: JobOccupationkindsInput,
    ) -> Operation:
        """
        Build a GraphQL mutation to update a job.

        :param job_id: The ID of the job to update.
        :param status: The status of the job.
        :param contract_kinds: A dictionary defining contract kinds to update.
        :param job_modes: A dictionary defining job modes to update.
        :param occupation_kinds: A dictionary defining occupation kinds to update.
        :return: An sgqlc Operation object representing the mutation.
        """
        op = Operation(RootMutation, name="update_job_mutation")
        job = op.update_job(
            input={
                "id": job_id,
                "status": status,
                "ignore_edit_lock": True,
                "contractkinds": contract_kinds,
                "jobmodes": job_modes,
                "occupationkinds": occupation_kinds,
            }
        ).job
        job.__fragment__(JobFragment().get())
        return op

    @staticmethod
    def parse(data: dict) -> UpdateJobMutationResponse:
        return UpdateJobMutationResponse.from_dict(data)


@dataclass
class UpdateJobStatusMutationResponse:
    job: Optional[Job]

    @staticmethod
    def from_dict(data: dict) -> "UpdateJobStatusMutationResponse":
        """
        Parse a dictionary into an UpdateJobStatusMutationResponse instance.
        """
        return UpdateJobStatusMutationResponse(
            job=Job.from_dict(data.get("job", {})) if data.get("job") else None
        )


class UpdateJobStatusMutation(BaseOperationBuilder):
    """
    A Builder for job update status GraphQL mutation
    """

    def get(self, id: ID, status: PostStatusEnum) -> Operation:
        """
        Build a GraphQL mutation to update a job.

        :param job_id: The ID of the job to update.
        :param status: The status of the job.
        :return: An sgqlc Operation object representing the mutation.
        """
        op = Operation(RootMutation, name="update_job_mutation")
        job = op.update_job(
            input={
                "id": job_id,
                "status": status,
                "ignore_edit_lock": True,
            }
        ).job
        job.__fragment__(JobFragment().get())
        return op

    @staticmethod
    def parse(data: dict) -> UpdateJobStatusMutationResponse:
        return UpdateJobStatusMutationResponse.from_dict(data)


@dataclass
class DeleteJobMutationResponse:
    job: Optional[Job]

    @staticmethod
    def from_dict(data: dict) -> "DeleteJobMutationResponse":
        """
        Parse a dictionary into a DeleteJobMutationResponse instance.
        """
        return DeleteJobMutationResponse(
            job=Job.from_dict(data.get("job", {})) if data.get("job") else None
        )


class DeleteJobMutation(BaseOperationBuilder):
    """
    A Builder for job update status GraphQL mutation
    """

    def get(self, id: ID) -> Operation:
        """
        Build a GraphQL mutation to update a job.

        :param job_id: The ID of the job to update.
        :return: An sgqlc Operation object representing the mutation.
        """

        op = Operation(RootMutation, name="update_job_mutation")
        job = op.delete_job(
            input={
                "id": job_id,
                "ignore_edit_lock": True,
                "force_delete": True,
            }
        ).job
        job.__fragment__(JobFragment().get())

        return op

    @staticmethod
    def parse(data: dict) -> DeleteJobMutationResponse:
        return DeleteJobMutationResponse.from_dict(data)



from dataclasses import dataclass, is_dataclass
from typing import TypeVar, Generic, List, Type, Union

T = TypeVar("T")


def _parse_dict(data: Union[dict, List[dict]], item_type: Type[T]) -> Union[T, List[T]]:
    """
    Parse a dictionary or list of dictionaries into an instance or list of instances of type T.
    If `item_type` is callable (e.g., a custom parsing function), it will use that directly.
    """

    def is_callable_type(item_type):
        return callable(item_type) and not is_dataclass(item_type)

    if isinstance(data, list):
        return [
            item_type(item)
            if is_callable_type(item_type)
            else item_type(
                **{
                    key: value
                    for key, value in item.items()
                    if key in item_type.__annotations__
                }
            )
            for item in data
        ]
    return (
        item_type(data)
        if is_callable_type(item_type)
        else item_type(
            **{
                key: value
                for key, value in data.items()
                if key in item_type.__annotations__
            }
        )
    )


@dataclass
class Node(Generic[T]):
    """
    Class to represent a single node.
    """

    node: T

    @staticmethod
    def from_dict(data: dict, item_type: Type[T]) -> "Node[T]":
        """
        Parse a dictionary into a single Node instance of type T.
        """
        return Node(node=_parse_dict(data.get("node", {}), item_type))


@dataclass
class Nodes(Generic[T]):
    """
    Class to represent multiple nodes.
    """

    nodes: List[T]

    @staticmethod
    def from_dict(data: dict, item_type: Type[T]) -> "Nodes[T]":
        """
        Parse a dictionary into a Nodes instance containing a list of items of type T.
        """
        return Nodes(nodes=_parse_dict(data.get("nodes", []), item_type))



from dataclasses import dataclass
from typing import Optional
from llq.type import Node, MediaItem


@dataclass
class PartnerAcf:
    partner_name: str
    partner_description: str
    partner_website_link: str
    technologie: list[str]
    type_of_company: list[str]
    partner_logo: Optional[Node[MediaItem]]


@dataclass
class Partner:
    id: str
    partner_acf: PartnerAcf

    @staticmethod
    def from_dict(data: dict) -> "Partner":
        partner_acf = PartnerAcf(
            partner_name=data["partner_acf"]["partner_name"],
            partner_description=data["partner_acf"]["partner_description"],
            partner_website_link=data["partner_acf"]["partner_website_link"],
            technologie=data["partner_acf"]["technologie"],
            type_of_company=data["partner_acf"]["type_of_company"],
            partner_logo=Node.from_dict(
                data["partner_acf"].get("partner_logo", {}), MediaItem
            )
            if data["partner_acf"].get("partner_logo")
            else None,
        )
        return Partner(id=data["id"], partner_acf=partner_acf)


__all__ = ["PartnerAcf", "Partner"]



from gql import Client as GqlClient, gql
from gql.transport.aiohttp import AIOHTTPTransport
from sgqlc.operation import Operation
from typing import Optional
from .base_client import BaseClient
from .exception import ClientException
import inflection


def to_snake_case(data):
    """
    Recursively converts dictionary keys to snake case.
    """
    if isinstance(data, dict):
        return {
            inflection.underscore(key): to_snake_case(value)
            for key, value in data.items()
        }
    elif isinstance(data, list):
        return [to_snake_case(item) for item in data]
    return data


class GraphQLClient(BaseClient):
    """
    A reusable asynchronous GraphQL client extending the BaseClient.
    """

    def __init__(self, endpoint_url: str, **kwargs):
        super().__init__(**kwargs)
        self.transport = AIOHTTPTransport(url=endpoint_url, headers=self.headers)
        self.client = GqlClient(transport=self.transport)
        self.session = None

    async def connect(self):
        self.session = await self.client.connect_async(reconnecting=True)

    async def close(self):
        if self.session:
            await self.client.close_async()

    async def execute(
        self, operation: Operation, variables: Optional[dict] = None
    ) -> dict:
        if not self.session:
            raise ClientException("Client is not connected. Call 'connect()' first.")

        gql_operation = gql(str(operation))
        response = await self.session.execute(gql_operation, variable_values=variables)
        return to_snake_case(response)



from sgqlc.operation import Operation, Fragment
from sgqlc.types import Variable
from llq.schema import (
    RootMutation,
    LoginInput,
    RefreshJwtAuthTokenInput,
)
from llq.base_operation_builder import BaseOperationBuilder
from dataclasses import dataclass


@dataclass
class LoginResponse:
    auth_token: str
    refresh_token: str

    @staticmethod
    def from_dict(data: dict) -> "LoginResponse":
        """
        Parse a dictionary into a LoginResponse instance.
        """
        return LoginResponse(
            auth_token=data["auth_token"],
            refresh_token=data["refresh_token"],
        )


class LoginMutation(BaseOperationBuilder):
    """
    A builder for login-related GraphQL mutation
    """

    def get(self, input: LoginInput) -> Operation:
        op = Operation(RootMutation, name="login_user_mutation")

        user = op.login(input=input)

        user.auth_token()
        user.refresh_token()

        return op

    @staticmethod
    def parse(data: dict) -> "LoginResponse":
        return LoginResponse.from_dict(data.get("login", {}))


@dataclass
class RefreshTokenResponse:
    auth_token: str

    @staticmethod
    def from_dict(data: dict) -> "RefreshTokenResponse":
        """
        Parse a dictionary into a RefreshTokenResponse instance.
        """
        return RefreshTokenResponse(auth_token=data["auth_token"])


class RefreshTokenMutation(BaseOperationBuilder):
    """
    A builder for refresh-token-related GraphQL mutation
    """

    def get(self, input: RefreshJwtAuthTokenInput) -> Operation:
        op = Operation(RootMutation, name="refresh_token_mutation")

        token = op.refresh_jwt_auth_token(input=input)

        token.auth_token()

        return op

    @staticmethod
    def parse(data: dict) -> "RefreshTokenResponse":
        return RefreshTokenResponse.from_dict(data.get("refreshJwtAuthToken", {}))



from sgqlc.operation import Operation, Fragment
from sgqlc.types import Variable, Int
from llq.schema import (
    RootQuery,
    Partner,
    ID,
)
from llq.fragments import PartnerFragment
from llq.base_operation_builder import BaseOperationBuilder
from dataclasses import dataclass
from typing import Any
from llq.base_response import BaseResponse
from llq.type import Nodes, Node, MediaItem
from llq.type.partner import PartnerAcf, Partner as PartnerType


@dataclass
class PartnersResponse(BaseResponse):
    partners: Nodes[PartnerType]

    @staticmethod
    def from_dict(data: dict) -> "PartnersResponse":
        """
        Parse a dictionary into a PartnersResponse instance.
        """
        return PartnersResponse(
            partners=Nodes.from_dict(data.get("partners", {}), PartnerType.from_dict)
        )


@dataclass
class PartnerByIdResponse(BaseResponse):
    partner: PartnerType

    @staticmethod
    def from_dict(data: dict) -> "PartnerByIdResponse":
        """
        Parse a dictionary into a PartnerByIdResponse instance.
        """
        return PartnerByIdResponse(
            partner=PartnerType.from_dict(data.get("partner", {}))
        )


class PartnerByIdQuery(BaseOperationBuilder):
    """
    A builder for partner-related GraphQL queries.
    """

    def get(self, id: ID) -> Operation:
        """
        Build a GraphQL query to fetch a partner by id.

        :param id: The ID of partner.
        :return: An sgqlc Operation object representing the query.
        """
        op = Operation(RootQuery, name="get_partner_by_id")

        partner = op.partner(id=id)
        partner.id()

        partner_acf = partner.partner_acf
        partner_frag = PartnerFragment().get()
        partner_acf.__fragment__(partner_frag)

        return op

    @staticmethod
    def parse(data: dict) -> PartnerByIdResponse:
        return PartnerByIdResponse.from_dict(data)


class PartnersQuery(BaseOperationBuilder):
    """
    A builder for partners-related GraphQL queries.
    """

    def get(self, first: Int) -> Operation:
        """
        Build a GraphQL query to fetch all partners.

        :param first: The number of partners to fetch.
        :return: An sgqlc Operation object representing the query.
        """

        op = Operation(RootQuery, name="get_partners")

        partners = op.partners(first=first)

        nodes = partners.nodes
        nodes.id()
        partner_node_frag = Fragment(Partner, "PartnerNodeFragment")
        partner_acf = partner_node_frag.partner_acf

        partner_acf_frag = PartnerFragment().get()
        partner_acf.__fragment__(partner_acf_frag)

        if partner_node_frag:
            nodes.__fragment__(partner_node_frag)

        return op

    @staticmethod
    def parse(data: dict) -> PartnersResponse:
        return PartnersResponse.from_dict(data)



from sgqlc.operation import Operation
from sgqlc.types import Variable, Int
from dataclasses import dataclass
from typing import Optional
from llq.schema import RootQuery
from llq.fragments import EventFragment, JobFragment
from llq.base_operation_builder import BaseOperationBuilder
from llq.base_response import BaseResponse
from llq.type.job import JobAcf, JobMode, OccupationKind, MediaItem, ContractKind
from llq.type import Nodes


@dataclass
class TermsResponse(BaseResponse):
    occupation_kinds: Nodes[OccupationKind]
    job_modes: Nodes[JobMode]
    contract_kinds: Nodes[ContractKind]

    @staticmethod
    def from_dict(data: dict) -> "TermsResponse":
        """
        Parse a dictionary into a TermsResponse instance.
        """
        return TermsResponse(
            occupation_kinds=Nodes.from_dict(
                data.get("occupationkinds", {}), OccupationKind
            ),
            job_modes=Nodes.from_dict(data.get("jobmodes", {}), JobMode),
            contract_kinds=Nodes.from_dict(data.get("contractkinds", {}), ContractKind),
        )


class CustomTermsQuery(BaseOperationBuilder):
    """
    A builder for job custom term GraphQL queries.
    """

    def get(
        self,
        first: Optional[Int] = 1,
    ) -> Operation:
        """
        Build a GraphQL query to fetch contract-kinds.

        :param contract_kind_count: The number of contract_kind to fetch.
        :param job_mode_count: The number of job_mode to fetch.
        :param occupation_kind_count: The number of occupation_kind to fetch.
        :return: An sgqlc Operation object representing the query.
        """
        op = Operation(RootQuery, name="get_job_custom_terms")

        contract_kinds = op.contractkinds(first=first)

        modes = op.jobmodes(first=first)

        occupation_kinds = op.occupationkinds(first=first)

        contract_kinds_nodes = contract_kinds.nodes.__fragment__(
            JobFragment().contract_kind_fragment()
        )

        job_modes_nodes = modes.nodes.__fragment__(JobFragment().job_mode_fragment())

        occupation_kinds_nodes = occupation_kinds.nodes.__fragment__(
            JobFragment().occupation_kind_fragment()
        )

        return op

    @staticmethod
    def parse(data: dict) -> TermsResponse:
        return TermsResponse.from_dict(data)



from sgqlc.operation import Fragment
from sgqlc.types import Type as SGQLCType
from typing import List
from llq.schema import Venue
from llq.fragments.fragment_builder import FragmentBuilder


class VenueFragment(FragmentBuilder[Venue]):
    def get(self) -> Fragment:
        # Venue Fragment
        fragment = Fragment(Venue, "VenueFragment")
        fragment.uri()
        fragment.city()
        fragment.address()

        return fragment



import aiohttp
from .base_client import BaseClient
from .exception import ClientException
from typing import Optional, Any, Dict


class RestClient(BaseClient):
    """
    A reusable asynchronous REST client extending the BaseClient.
    """

    def __init__(self, base_url: str, **kwargs):
        super().__init__(**kwargs)
        self.base_url = base_url.rstrip("/")
        self.session: Optional[aiohttp.ClientSession] = None

    async def connect(self):
        if not self.session:
            self.session = aiohttp.ClientSession(headers=self.headers)

    async def close(self):
        if self.session:
            await self.session.close()
            self.session = None

    async def request(self, method: str, endpoint: str, **kwargs) -> dict:
        if not self.session:
            raise ClientException("Client is not connected. Call 'connect()' first.")

        url = f"{self.base_url}/{endpoint.lstrip('/')}"
        self.log_request(method, url, **kwargs)

        @self.retry_request
        async def _make_request():
            async with self.session.request(method, url, **kwargs) as response:
                response.raise_for_status()
                data = await response.json()
                self.log_response(response.status, data)
                return data

        return await _make_request()

    async def get(self, endpoint: str, params: Optional[Dict] = None) -> dict:
        return await self.request("GET", endpoint, params=params)

    async def post(
        self, endpoint: str, data: Optional[Any] = None, json: Optional[Any] = None
    ) -> dict:
        return await self.request("POST", endpoint, data=data, json=json)

    async def put(
        self, endpoint: str, data: Optional[Any] = None, json: Optional[Any] = None
    ) -> dict:
        return await self.request("PUT", endpoint, data=data, json=json)

    async def delete(self, endpoint: str, params: Optional[Dict] = None) -> dict:
        return await self.request("DELETE", endpoint, params=params)



from sgqlc.operation import Operation
from sgqlc.types import Variable, Int
from llq.schema import (
    RootQuery,
    Event,
    DateQueryInput,
    OrderEnum,
    PostObjectsConnectionOrderbyEnum,
)
from llq.fragments import EventFragment
from llq.base_operation_builder import BaseOperationBuilder
from llq.type import Nodes
from llq.type.event import Event
from llq.base_response import BaseResponse
from dataclasses import dataclass


@dataclass
class EventByStartDateResponse(BaseResponse):
    events: Nodes[Event]

    @staticmethod
    def from_dict(data: dict) -> "EventByStartDateResponse":
        """
        Parse a dictionary response into an EventByStartDateResponse instance.
        """
        return EventByStartDateResponse(
            events=Nodes.from_dict(data.get("events", {}), Event.from_dict)
        )


class EventByStartDateQuery(BaseOperationBuilder):
    """
    A builder for event-related GraphQL queries.
    """

    def get(self, first: Int, date: DateQueryInput) -> Operation:
        """
        Build a GraphQL query to fetch the next events.

        :param first: The number of events to fetch.
        :param start_date: The starting date for the events.
        :return: An sgqlc Operation object representing the query.
        """
        op = Operation(RootQuery, name="get_next_event")

        events = op.events(
            first=first,
            where={
                "date_query": date,
                "order_by": {
                    "field": PostObjectsConnectionOrderbyEnum.DATE,
                    "order": OrderEnum.ASC,
                },
            },
        )

        nodes = events.nodes

        # Retrieve and attach the EventFragment from the registry
        event_frag = EventFragment().get()
        nodes.__fragment__(event_frag)

        return op

    @staticmethod
    def parse(data: dict) -> EventByStartDateResponse:
        return EventByStartDateResponse.from_dict(data)



from .event import EventByStartDateQuery
from .partner import PartnerByIdQuery, PartnersQuery
from .job import CustomTermsQuery

__all__ = [
    "EventByStartDateQuery",
    "PartnerByIdQuery",
    "PartnersQuery",
    "CustomTermsQuery",
]



from uuid_utils import uuid7
from dataclasses import dataclass, asdict
from typing import Any, Dict
from llq.clients import RestClient
from llq.schema import PostStatusEnum
from llq.type.job import CreateJobAcf


async def post_job(job: CreateJobAcf, client: RestClient) -> Dict:
    job_post = {
        "title": job.job_title_,
        "status": PostStatusEnum.DRAFT.lower(),
        "slug": str(uuid7()),
        "acf": asdict(job),
    }
    return await client.post("/job", json=job_post)



from .event import EventFragment
from .organizer import OrganizerFragment
from .venue import VenueFragment
from .partner import PartnerFragment
from .media_item import MediaItemFragment
from .job import JobFragment

__all__ = [
    "EventFragment",
    "OrganizerFragment",
    "VenueFragment",
    "PartnerFragment",
    "MediaItemFragment",
    "JobFragment",
]



from abc import ABC, abstractmethod
from typing import TypeVar, Generic
from sgqlc.operation import Fragment
from sgqlc.types import Type as SGQLCType

T = TypeVar("T", bound=SGQLCType)


class FragmentBuilder(ABC, Generic[T]):
    """
    Abstract base class for building GraphQL fragment.
    """

    @abstractmethod
    def get(self) -> Fragment:
        """
        Build and return a of GraphQL fragment.

        :return: A sgqlc Fragment object.
        """
        pass



from sgqlc.operation import Fragment
from sgqlc.types import Type as SGQLCType
from typing import List
from llq.schema import Organizer, MediaItem
from llq.fragments.fragment_builder import FragmentBuilder
from llq.fragments.media_item import MediaItemFragment


class OrganizerFragment(FragmentBuilder[Organizer]):
    def get(self) -> Fragment:
        # Organizer Fragment
        fragment = Fragment(Organizer, "OrganizerFragment")
        fragment.uri()
        fragment.title()
        fragment.content()

        # Nested MediaItem Fragment
        media_item_frag = MediaItemFragment().get()
        image = fragment.featured_image
        image_node = image.node
        image_node.__fragment__(media_item_frag)

        return fragment



from .auth import LoginMutation, RefreshTokenMutation
from .job import UpdateJobMutation, UpdateJobStatusMutation, DeleteJobMutation

__all__ = [
    "LoginMutation",
    "RefreshTokenMutation",
    "UpdateJobStatusMutation",
    "UpdateJobMutation",
    "DeleteJobMutation",
]



"""Generate the code reference pages and navigation."""

import importlib

from pathlib import Path

import mkdocs_gen_files

nav = mkdocs_gen_files.Nav()

root = Path(__file__).parent.parent

modules = mkdocs_gen_files.config["extra"]["modules"]

for module in modules:
    module_path = Path(*module.split(".")) / "__init__.py"
    m = importlib.import_module(module)

    for item in m.__all__:
        doc_path = Path("reference", module, item)
        file_path = doc_path.with_suffix(".md")
        nav[(module, item)] = Path(module, item).with_suffix(".md").as_posix()
        with mkdocs_gen_files.open(file_path, "w") as fd:
            fd.write(f"::: {module}.{item}")
        mkdocs_gen_files.set_edit_path(file_path, module_path)

with mkdocs_gen_files.open("reference/SUMMARY.md", "w") as nav_file:
    nav_file.writelines(nav.build_literate_nav())



from sgqlc.operation import Fragment
from llq.schema import JobAcf, Job, Occupationkind, Jobmode, ContractKind
from llq.fragments.fragment_builder import FragmentBuilder
from llq.fragments.media_item import MediaItemFragment


class JobFragment(FragmentBuilder[JobAcf]):
    def get(self) -> Fragment:
        """
        Defines the JobFragment.

        :return: An sgqlc Fragment object.
        """
        fragment = Fragment(Job, "JobFragment")
        fragment.uri()

        # Nested Fragments
        job_acf = fragment.job_acf()
        job_acf.__fragment__(self._job_acf_fragment())

        occupation_kind_nodes = fragment.occupation_kinds().nodes
        occupation_kind_nodes.__fragment__(self.occupation_kind_fragment())

        job_mode_nodes = fragment.job_modes().nodes
        job_mode_nodes.__fragment__(self.job_mode_fragment())

        contract_kind_nodes = fragment.contract_kinds().nodes
        contract_kind_nodes.__fragment__(self.contract_kind_fragment())

        return fragment

    def _job_acf_fragment(self) -> Fragment:
        fragment = Fragment(JobAcf, "JobAcfFragment")
        fragment.job_title()
        fragment.compagny_name()
        fragment.localization()
        fragment.apply_link()

        image_node = fragment.compagny_logo().node
        media_item_frag = MediaItemFragment().get()
        image_node.__fragment__(media_item_frag)

        return fragment

    def occupation_kind_fragment(self) -> Fragment:
        fragment = Fragment(Occupationkind, "OccupationKindFragment")
        fragment.id()
        fragment.uri()
        fragment.name()
        return fragment

    def job_mode_fragment(self) -> Fragment:
        fragment = Fragment(Jobmode, "JobModeFragment")
        fragment.id()
        fragment.uri()
        fragment.name()
        return fragment

    def contract_kind_fragment(self) -> Fragment:
        fragment = Fragment(ContractKind, "ContractKindFragment")
        fragment.id()
        fragment.uri()
        fragment.name()
        return fragment



import logging
import backoff
from typing import Optional, Dict


class BaseClient:
    """
    A base class for shared functionality between GraphQL and REST clients.
    """

    def __init__(
        self,
        headers: Optional[Dict[str, str]] = None,
        retry_connect=None,
        retry_request=None,
    ):
        """
        Initialize the base client with common attributes.

        :param headers: Optional headers for HTTP requests.
        :param retry_connect: Custom retry logic for connection retries.
        :param retry_request: Custom retry logic for request retries.
        """
        self.headers = headers or {}
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.setLevel(logging.DEBUG)
        self.retry_connect = retry_connect or backoff.on_exception(
            backoff.expo, Exception, max_value=60
        )
        self.retry_request = retry_request or backoff.on_exception(
            backoff.expo, Exception, max_tries=5
        )

    def log_request(self, method: str, url: str, **kwargs):
        """
        Log the details of an HTTP request.
        """
        self.logger.debug(f"Requesting {method.upper()} {url} with {kwargs}")

    def log_response(self, status: int, response: str):
        """
        Log the details of an HTTP response.
        """
        self.logger.debug(f"Response status: {status}, response: {response}")



from sgqlc.operation import Fragment
from llq.schema import MediaItem
from llq.fragments.fragment_builder import FragmentBuilder


class MediaItemFragment(FragmentBuilder[MediaItem]):
    def get(self) -> Fragment:
        """
        Defines the MediaItemFragment.

        :return: An sgqlc Fragment object.
        """
        fragment = Fragment(MediaItem, "MediaItemFragment")
        fragment.uri()
        fragment.source_url()
        fragment.alt_text()
        fragment.database_id()
        fragment.media_item_url()
        return fragment
